/*
 * Attack.cxx
 * Daniel Nelson - 8/29/0
 *
 * Copyright (C) 2000  Daniel Nelson
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Daniel Nelson - aluminumangel.org
 * 174 W. 18th Ave.
 * Columbus, OH  43210
 */
     
#include <GL/glut.h>
#include <string.h>

#include "Attack.h"
#include "Communicator.h"
#include "Game.h"
#include "MetaState.h"
#include "Random.h"

// documentation:
//   name escape codes: ~<>123...
//   my_tga - 32 color depth, uncompressed, size max, deletion

/*
 * Issues and Watches
 *   velocity increases instead of period decreases?
 *   reseed Random after setup
 *   net hangs / previous time changing ?
 *   windows version (including OpenGL 1.2)
 *   tar makes own directory
 */

int main ( int argc, char **argv )
{
  char player_name[GC_PLAYER_NAME_LENGTH];
  char host_name[256];
  int port;
  int mode = 0;

  glutInit(&argc, argv);

  player_name[0] = '\0';
  parseCommandLine(argc, argv, mode, port, host_name, player_name);
  
  if (player_name[0] == '\0')
    strncpy(player_name, GC_DEFAULT_PLAYER_NAME, GC_PLAYER_NAME_LENGTH);

  cout << GC_MESSAGE << endl;

  if (!(mode & CM_SOLO))
    Communicator::initialize(mode, port, host_name, player_name);
  else
    Random::seed(Random::generateSeed());

  MetaState::programStart(mode, player_name);

  glutMainLoop();
}

void parseCommandLine ( int argc, char **argv, int &mode, int &port,
 char *host_name, char *player_name )
{
  for (int n = 1; argv[n]; n++) {

    if (!strcmp(argv[n], "-s") || !strcmp(argv[n], "--server")) {
      if (mode & (CM_SERVER | CM_CLIENT | CM_SOLO)) usage();

      mode |= CM_SERVER;
      if (argv[n + 1] && argv[n + 1][0] != '-')
        port = atoi(argv[++n]);
      else
        port = 0;

    } else if (!strcmp(argv[n], "-1") || !strcmp(argv[n], "--solo")) {
      if (mode & (CM_SERVER | CM_CLIENT | CM_SOLO)) usage();

      mode |= CM_SOLO;

    } else if (!strcmp(argv[n], "-n") || !strcmp(argv[n], "--name")) {
      if (!argv[n + 1]) usage();

      strncpy(player_name, argv[++n], GC_PLAYER_NAME_LENGTH - 1);
      player_name[GC_PLAYER_NAME_LENGTH - 1] = '\0';

    } else if (!strcmp(argv[n], "-X") || !strcmp(argv[n], "--extreme"))

      mode |= CM_X;

    else if (!strcmp(argv[n], "-w") || !strcmp(argv[n], "--wait"))
      mode |= CM_NO_TIME_OUT;

    else {
      if (mode & (CM_SERVER | CM_CLIENT | CM_SOLO)) usage();

      mode |= CM_CLIENT;
      strcpy(host_name, argv[n]);
      char *ptr = strchr(host_name, ':');
      if (ptr) {
        port = atoi(ptr + 1);
        *ptr = '\0';
      } else
        port = 0;
    }
  }

  if (!(mode & (CM_SERVER | CM_CLIENT | CM_SOLO)))
    usage();

  if ((mode & CM_NO_TIME_OUT) && !(mode & CM_SERVER))
    usage();
}
