/*
 * DrawExternalCandy.cxx
 * Daniel Nelson - 10/13/0
 *
 * Copyright (C) 2000  Daniel Nelson
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Daniel Nelson - aluminumangel.org
 * 174 W. 18th Ave.
 * Columbus, OH  43210
 *
 * Draws everything dumb outside of the play area.
 */

#include <GL/glut.h>

#include "Game.h"
#include "Displayer.h"
#include "Clock.h"

void inline Displayer::drawDigit ( GLfloat alpha )
{
  glColor4f(0.3f, 0.3f, 1.0f, alpha);

  glBegin(GL_TRIANGLE_STRIP);

    glTexCoord2f(0.0f, 1.0f);
    glVertex3f(-DC_CLOCK_DIGIT_LENGTH, -DC_CLOCK_DIGIT_LENGTH, 0.0f);
    glColor4f(0.5f, 0.5f, 1.0f, alpha);
    glTexCoord2f(1.0f, 1.0f);
    glVertex3f(DC_CLOCK_DIGIT_LENGTH, -DC_CLOCK_DIGIT_LENGTH, 0.0f);
    glColor4f(0.5f, 0.5f, 1.0f, alpha);
    glTexCoord2f(0.0f, 0.0f);
    glVertex3f(-DC_CLOCK_DIGIT_LENGTH, DC_CLOCK_DIGIT_LENGTH, 0.0f);
    glColor4f(1.0f, 1.0f, 1.0f, alpha);
    glTexCoord2f(1.0f, 0.0f);
    glVertex3f(DC_CLOCK_DIGIT_LENGTH, DC_CLOCK_DIGIT_LENGTH, 0.0f);

  glEnd();
}

void Displayer::drawExternalCandy (   )
{
  // draw clock

  int texture_bound;

  glPushMatrix();

    glTranslatef(DC_LEFT_EXTERNAL_CENTER - DC_CLOCK_DIGIT_OFFSET / 2,
     DC_CLOCK_OFFSET_Y, DC_EXTERNAL_OFFSET_Z);

    glBindTexture(GL_TEXTURE_2D, clock_digit_textures[10]);
    drawDigit(1.0f);
    
    glTranslatef(DC_CLOCK_CENTER_OFFSET / 2.0f
     + 3.0f * DC_CLOCK_DIGIT_OFFSET / 2.0f, 0.0f, 0.0f);

    GLfloat alpha = Clock::tick * (1.0f / GC_STEPS_PER_SECOND);

    glBindTexture(GL_TEXTURE_2D, clock_digit_textures[Clock::digits[0]]);
    if (Clock::digits[0] != Clock::previous_digits[0]) {
      drawDigit(alpha);
      glBindTexture(GL_TEXTURE_2D,
       clock_digit_textures[Clock::previous_digits[0]]);
      drawDigit(1.0f - alpha);
      texture_bound = Clock::previous_digits[0];
    } else {
      drawDigit(1.0f);
      texture_bound = Clock::digits[0];
    }

    glTranslatef(-DC_CLOCK_DIGIT_OFFSET, 0.0f, 0.0f);

    if (Clock::digits[1] != texture_bound)
      glBindTexture(GL_TEXTURE_2D, clock_digit_textures[Clock::digits[1]]);
    if (Clock::digits[1] == Clock::previous_digits[1]) {
      drawDigit(1.0f);
      texture_bound = Clock::digits[1];
    } else {
      drawDigit(alpha);
      glBindTexture(GL_TEXTURE_2D,
       clock_digit_textures[Clock::previous_digits[1]]);
      drawDigit(1.0f - alpha);
      texture_bound = Clock::previous_digits[1];
    }

    glTranslatef(-DC_CLOCK_DIGIT_OFFSET - DC_CLOCK_CENTER_OFFSET, 0.0f, 0.0f);

    if (Clock::digits[2] != texture_bound)
      glBindTexture(GL_TEXTURE_2D, clock_digit_textures[Clock::digits[2]]);
    if (Clock::digits[2] == Clock::previous_digits[2]) {
      drawDigit(1.0f);
      texture_bound = Clock::digits[2];
    } else {
      drawDigit(alpha);
      glBindTexture(GL_TEXTURE_2D,
       clock_digit_textures[Clock::previous_digits[2]]);
      drawDigit(1.0f - alpha);
      texture_bound = Clock::previous_digits[2];
    }

    if (Clock::digits[3] || Clock::previous_digits[3]) {
      glTranslatef(-DC_CLOCK_DIGIT_OFFSET, 0.0f, 0.0f);

      if (Clock::digits[3] == Clock::previous_digits[3]) {
        if (texture_bound != Clock::digits[3])
          glBindTexture(GL_TEXTURE_2D, clock_digit_textures[Clock::digits[3]]);
        drawDigit(1.0f);
      } else {
        if (Clock::digits[3]) {
          if (texture_bound != Clock::digits[3])
            glBindTexture(GL_TEXTURE_2D,
             clock_digit_textures[Clock::digits[3]]);
          drawDigit(alpha);
          texture_bound = Clock::digits[3];
        }
        if (Clock::previous_digits[3]) {
          if (texture_bound != Clock::previous_digits[3])
            glBindTexture(GL_TEXTURE_2D,
             clock_digit_textures[Clock::previous_digits[3]]);
          drawDigit(1.0f - alpha);
        }
      }
    }

  glPopMatrix();

  glDisable(GL_BLEND);

  // draw logo

  glBindTexture(GL_TEXTURE_2D, logo_texture);
  glCallList(logo_list);

  // draw names

  glPushMatrix();
    glBindTexture(GL_TEXTURE_2D, name_texture);
    glTranslatef(DC_LEFT_EXTERNAL_CENTER, DC_NAME_OFFSET_Y,
     DC_EXTERNAL_OFFSET_Z);
    glScalef(DC_NAME_SCALE, DC_NAME_SCALE, 1.0f);
    glCallList(message_2x1_list);
  glPopMatrix();
}
